﻿/*
create by:dint 
date:2014.12.12
http://www.cnblogs.com/dint/
*/
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using System.Threading;
using System.Reflection;
using System.Reflection.Emit;
using System.Data.OracleClient;
using System.Data.SqlClient;

namespace DBMapper
{
    public class DbMapper {
        internal static char[] SPLIT_SQL_PARAMS_IDENTY= new char[] { ':' };
        internal readonly static char[] SPLIT_TABLE_PARAM_IDENTY = new char[] { '#' };
        internal static char CHAR_SQL_PARAMS_IDENTY = ':';
        internal readonly static char CHAR_TABLE_PARAM_IDENTY = '#';
        public readonly static string PROVIDER_ORACLE= "System.Data.OracleClient";
        public readonly static string PROVIDER_SQLSERVER = "System.Data.SqlClient";
        public static IDbMapper CreateDbMapper(string dbProvider, string dbLink)
        {
            DbMapperPrivder dbMapper = null;
            if (dbProvider==PROVIDER_ORACLE)
            {
                dbMapper = new OracleDbMapper();
                dbMapper.Connection(dbLink);
            }
            if (dbProvider == PROVIDER_SQLSERVER)
            {
                dbMapper = new SqlServerDbMapper();
                dbMapper.Connection(dbLink);
                SPLIT_SQL_PARAMS_IDENTY = new char[] { '@' };
                CHAR_SQL_PARAMS_IDENTY = '@';
            }
            return (IDbMapper)dbMapper;
        }
    }
    
    /// <summary>
    ///可查询对象 提供一组数据库查询结果的操作 默认操作类型为T
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public interface IDbQuery<T> : IEnumerable<T>
    {
        /// <summary>
        /// 获取第一个元素
        /// </summary>
        /// <returns>T类型元素</returns>
        T First { get; }
        /// <summary>
        /// 获取TReturn类型元素集合
        /// </summary>
        /// <typeparam name="TReturn">返回类型</typeparam>
        /// <returns>TReturn类型的集合</returns>
        List<TReturn> ToList<TReturn>() where TReturn : new();
        IEnumerator<TReturn> GetEnumerator<TReturn>() where TReturn : new();
        /// <summary>
        /// 获取T类型元素集合
        /// </summary>
        /// <returns>T类型元素的集合</returns>
        List<T> ToList();
        /// <summary>
        /// 获取元素个数
        /// </summary>
        int Count { get; }
        /// <summary>
        /// 获取DataTable形式的查询结果
        /// </summary>
        /// <returns>DataTable形式的查询结果</returns>
        DataTable ToTable();
        /// <summary>
        ///  跳到某一元素
        /// </summary>
        /// <param name="index">元素索引</param>
        /// <returns></returns>
        IDbQuery<T> Skip(int index);
        /// <summary>
        /// 获取当前元素后面的多少个元素
        /// </summary>
        /// <param name="count">元素个数</param>
        /// <returns></returns>
        IDbQuery<T> Take(int count);
        /// <summary>
        /// 排序
        /// </summary>
        /// <param name="name">名称</param>
        /// <param name="direct">方向 True 为顺序排序 False为倒序排序</param>
        /// <returns></returns>
        IDbQuery<T> OrderBy(string name, bool direct);
    }
    
    /// <summary>
    /// 提供一组数据库执行的操作
    /// </summary>
    public interface IDbExecute : IDisposable
    {
        /// <summary>
        /// 执行非查询操作
        /// </summary>
        /// <param name="bTransaction">是否在事务中处理</param>
        /// <returns>影响的行数</returns>
        int Execute(bool bTransaction);
        /// <summary>
        /// 执行无事务的非查询操作
        /// </summary>
        /// <returns>影响的行数</returns>
        int Execute();
        /// <summary>
        /// 执行查询操作
        /// </summary>
        /// <returns>查询的结果</returns>
        DataTable ExecuteQuery();
        /// <summary>
        /// 查询单个数据
        /// </summary>
        /// <returns>第一行第一列数据</returns>
        object ExecuteScalar();
        /// <summary>
        /// 查询单行数据
        /// </summary>
        /// <returns>行数据以 列名/值 的键值对保存</returns>
        Dictionary<string, object> ExecuteReader();
    }
    
    /// <summary>
    /// 提供动态数据库参数操作的一组方法
    /// </summary>
    public interface IDbDynParameters
    {
        /// <summary>
        /// 获取一个动态参数的值
        /// </summary>
        /// <typeparam name="T">参数类型</typeparam>
        /// <param name="name">参数名称</param>
        /// <returns>参数值</returns>
        T Get<T>(string name);
        /// <summary>
        /// 添加一个动态参数
        /// </summary>
        /// <typeparam name="T">参数类型</typeparam>
        /// <param name="name">参数名称</param>
        /// <param name="value">参数值</param>
        void Add<T>(string name, T value);
        /// <summary>
        /// 添加一个动态参数
        /// </summary>
        /// <typeparam name="T">参数类型</typeparam>
        /// <param name="name">参数名称</param>
        /// <param name="value">参数值</param>
        /// <param name="direction">参数方向(输入/输出)</param>
        void Add<T>(string name, T value, ParameterDirection direction);
        /// <summary>
        /// 添加一个动态参数
        /// </summary>
        /// <typeparam name="T">参数类型</typeparam>
        /// <param name="name">参数名称</param>
        /// <param name="value">参数值</param>
        /// <param name="direction">参数方向(输入/输出)</param>
        /// <param name="size">参数大小</param>
        void Add<T>(string name, T value, ParameterDirection direction, int size);
    }
   
    /// <summary>
    /// DBMapper可提供的操作
    /// </summary>
    public interface IDbMapper : IDbExecute
    {

        #if NET4_0 //.net4.0以上版本 才支持动态对象
        /// <summary>
        /// 定义一个查询操作
        /// </summary>
        /// <typeparam name="T">可查询操作的默认返回类型</typeparam>
        /// <param name="sql">查询语句</param>
        /// <param name="entity">参数</param>
        /// <returns>可查询对象</returns>
        IDbQuery<T> Query<T>(string sql, dynamic entity)
            where T : class,new();
        #endif
        /// <summary>
        /// 定义一个查询操作
        /// </summary>
        /// <typeparam name="T">查询操作涉及的对象/数据库表实体</typeparam>
        /// <param name="sql">查询语句</param>
        /// <param name="entity">参数</param>
        /// <returns>可查询对象</returns>
        IDbQuery<T> Query<T>(string sql, T entity)
            where T : class, new();
        /// <summary>
        /// 定义一个查询操作
        /// </summary>
        /// <typeparam name="T">查询操作涉及的对象/数据库表实体</typeparam>
        /// <typeparam name="T1">查询操作涉及的对象/数据库表实体</typeparam>
        /// <param name="sql">查询语句</param>
        /// <param name="entity">参数</param>
        /// <param name="entity1">参数</param>
        /// <returns>可查询对象</returns>
        IDbQuery<T> Query<T, T1>(string sql, T entity, T1 entity1)
            where T : class, new()
            where T1 : class, new();
        IDbQuery<T> Query<T, T1, T2>(string sql, T entity, T1 entity1, T2 entity2)
            where T : class,new()
            where T1 : class, new()
            where T2 : class, new();
        IDbQuery<T> Query<T, T1, T2, T3>(string sql, T entity, T1 entity1, T2 entity2, T3 entity3)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new();
        IDbQuery<T> Query<T, T1, T2, T3, T4>(string sql, T entity, T1 entity1, T2 entity2, T3 entity3, T4 entity4)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new()
            where T4 : class, new();
        IDbQuery<T> Query<T, T1, T2, T3, T4, T5>(string sql, T entity, T1 entity1, T2 entity2, T3 entity3, T4 entity4, T5 entity5)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new()
            where T4 : class, new()
            where T5 : class, new();
        IDbQuery<T> Query<T, T1, T2, T3, T4, T5, T6>(string sql, T entity, T1 entity1, T2 entity2,
            T3 entity3, T4 entity4, T5 entity5, T6 entity6)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new()
            where T4 : class, new()
            where T5 : class, new()
            where T6 : class, new();
        /// <summary>
        ///  定义对某个表的非查询操作（增,删,改）
        /// </summary>
        /// <typeparam name="T">操作的表实体</typeparam>
        /// <param name="sql">sql语句</param>
        /// <param name="entity">实体</param>
        void Action<T>(string sql, T entity);
        /// <summary>
        /// 定义一个非查询操作（增，删，改，存储过程）
        /// </summary>
        /// <param name="sql">SQL语句/过程名称</param>
        /// <param name="parameters">参数列表</param>
        void Action(string sql, IDbDynParameters parameters);
        /// <summary>
        /// 定义对某张表的修改操作
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="entity">实体</param>
        void Update<T>(T entity);
        /// <summary>
        /// 定义对某张表的修改操作
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="entity">实体</param>
        void Insert<T>(T entity);
        /// <summary>
        /// 定义对某张表的删除操作
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="entity">实体</param>
        void Delete<T>(T entity);
    }

    /// <summary>
    /// SQL
    /// </summary>
    public interface ISQLEntity
    {
        /// <summary>
        ///  语句
        /// </summary>
        string SQL { get; }
        /// <summary>
        /// 参数
        /// </summary>
        List<DbParameter> Parameters { get; }
        /// <summary>
        /// 操作类型
        /// </summary>
        SQLType SqlType { get; }
    }
    
    /// <summary>
    /// 数据库表名称映射
    /// </summary>
    public interface IDbTableNaming
    {
        /// <summary>
        /// 该实体对应的数据库表名
        /// </summary>
        string DBTableName { get; }
    }
    internal interface _IDbExecute
    {
        int Execute(ISQLEntity aSQL);
        DataTable ExecuteQuery(ISQLEntity aSQL);
        object ExecuteScalar(ISQLEntity aSQL);
        Dictionary<string, object> ExecuteReader(ISQLEntity aSQL);
    }
    
    /// <summary>
    ///指示列是否为主键
    /// </summary>
    [AttributeUsage(AttributeTargets.Property)]
    public class PrimaryKeyAttribute : Attribute {
        public static Type CLASS_TYPE = typeof(PrimaryKeyAttribute);
    }
    
    /// <summary>
    /// SQL操作类型
    /// </summary>
    public enum SQLType
    {
        Select,
        Insert,
        Update,
        Delete,
        StoreProc
    }
    
    /// <summary>
    /// 提供DBMapper的基础实现
    /// </summary>
    public abstract class DbMapperPrivder:IDbExecute,_IDbExecute
    {
        protected IList<ISQLEntity> dbSQLs = null;
        protected DbProviderFactory _Dbfactory = null;
        protected DbConnection _DbConnection = null;
        protected DbCommand _DbCommand = null;
        protected DbDataAdapter _DbDataAdapter = null;
        protected DbMapperPrivder() {
            dbSQLs = new List<ISQLEntity>();
        }
        public abstract void Connection(string connString);
        protected virtual void InitConnection(string dbPrivder) {
            _Dbfactory = DbProviderFactories.GetFactory(dbPrivder);
            _DbConnection = _Dbfactory.CreateConnection();
            _DbCommand = _Dbfactory.CreateCommand();
            _DbDataAdapter = _Dbfactory.CreateDataAdapter();
        }
        public virtual int Execute(bool bTransaction) {
            _DbCommand.Connection = _DbConnection;
            _DbCommand.CommandType = CommandType.Text;
            if (_DbConnection.State == ConnectionState.Broken)
                _DbConnection.Close();
            if (_DbConnection.State == ConnectionState.Closed)
                _DbConnection.Open();
            DbTransaction aTransaction = null;
            int affected = 0;
            try
            {
                if (bTransaction){
                    aTransaction = _DbConnection.BeginTransaction(IsolationLevel.ReadCommitted);
                    _DbCommand.Transaction = aTransaction;
                }
                foreach (ISQLEntity aSQL in dbSQLs)
                {
                    if ((aSQL.SqlType == SQLType.Insert)
                        || (aSQL.SqlType == SQLType.Update)
                        || (aSQL.SqlType == SQLType.Delete))
                    {
                        _DbCommand.Parameters.Clear();
                        _DbCommand.CommandText = aSQL.SQL;
                        foreach (DbParameter aParam in aSQL.Parameters)
                            _DbCommand.Parameters.Add(aParam);

                        affected += _DbCommand.ExecuteNonQuery();
                    } 
                }
                dbSQLs.Clear();
                if (aTransaction != null) aTransaction.Commit();
            }
            catch{
                if (aTransaction != null) aTransaction.Rollback();
                throw;
            }
            finally{
                _DbConnection.Close();
            }
            return affected;
        }
        public virtual int Execute()
        {
            int affected = 0;            
            foreach (ISQLEntity aSQL in dbSQLs) {
               affected+=Execute(aSQL);
            }
            dbSQLs.Clear();
            return affected;
        }
        protected virtual int Execute(ISQLEntity aSQL) {
            if (aSQL == null)
                throw new ArgumentNullException("sql entity is null");
            _DbCommand.Connection = _DbConnection;
            switch (aSQL.SqlType) { 
                case SQLType.Insert:
                case SQLType.Update:
                case SQLType.Delete:
                    _DbCommand.CommandType = CommandType.Text;
                    break;
                case SQLType.StoreProc:
                    _DbCommand.CommandType = CommandType.StoredProcedure;
                    break;
                default:
                    throw new ArgumentException("sql type error");
            }
            _DbCommand.CommandType = CommandType.Text;
            if (_DbConnection.State == ConnectionState.Broken)
                _DbConnection.Close();
            if (_DbConnection.State == ConnectionState.Closed)
                _DbConnection.Open();
            try
            {
                _DbCommand.Parameters.Clear();
                _DbCommand.CommandText = aSQL.SQL;
                foreach (DbParameter aParam in aSQL.Parameters)
                    _DbCommand.Parameters.Add(aParam);
                return _DbCommand.ExecuteNonQuery();
            }
            finally
            {
                _DbConnection.Close();
            }
        } 
        public virtual DataTable ExecuteQuery() {
            if (dbSQLs.Count > 0){
                return ExecuteQuery(dbSQLs[0]);
             }
            return null;
        }
        protected virtual DataTable ExecuteQuery(ISQLEntity aSQL) {
            if (aSQL == null)
                throw new ArgumentNullException("sql entity is null");
            if (aSQL.SqlType != SQLType.Select)
                throw new ArgumentException("sql type error");
            _DbDataAdapter.SelectCommand = _DbCommand;
            _DbCommand.Connection = _DbConnection;
            _DbCommand.CommandType = CommandType.Text;
            if (_DbConnection.State == ConnectionState.Broken)
                _DbConnection.Close();
            if (_DbConnection.State == ConnectionState.Closed)
                _DbConnection.Open();
            try
            {
                _DbCommand.Parameters.Clear();
                _DbCommand.CommandText = aSQL.SQL;
                foreach (DbParameter aParam in aSQL.Parameters)
                    _DbCommand.Parameters.Add(aParam);
                DataTable aTable = new DataTable();
                _DbDataAdapter.Fill(aTable);
                dbSQLs.Clear();
                return aTable;
            }
            finally
            {
                _DbConnection.Close();
            }

        }
        public virtual object ExecuteScalar() {
           return ExecuteScalar(dbSQLs[0]);
        }
        protected virtual object ExecuteScalar(ISQLEntity aSQL)
        {
            if (aSQL == null)
                throw new ArgumentNullException("sql entity is null");
            if (aSQL.SqlType != SQLType.Select) 
                throw new ArgumentException("sql type error");
            _DbCommand.Connection = _DbConnection;
            _DbCommand.CommandType = CommandType.Text;
            if (_DbConnection.State == ConnectionState.Broken)
                _DbConnection.Close();
            if (_DbConnection.State == ConnectionState.Closed)
                _DbConnection.Open();
            try
            {
                _DbCommand.Parameters.Clear();
                _DbCommand.CommandText = aSQL.SQL;
                foreach (DbParameter aParam in aSQL.Parameters)
                    _DbCommand.Parameters.Add(aParam);
                object oValue = _DbCommand.ExecuteScalar();
                dbSQLs.Clear();
                return oValue;
            }
            finally
            {
                _DbConnection.Close();
            }
        }
        public virtual Dictionary<string, object> ExecuteReader(){
            return ExecuteReader(dbSQLs[0]);
        }
        protected virtual Dictionary<string,object> ExecuteReader(ISQLEntity aSQL) {
            if (aSQL == null) 
                throw new ArgumentNullException("sqlentity is null");
            if (aSQL.SqlType != SQLType.Select)
                throw new ArgumentException("sql type error");
            _DbCommand.Connection = _DbConnection;
            _DbCommand.CommandType = CommandType.Text;
            if (_DbConnection.State == ConnectionState.Broken)
                _DbConnection.Close();
            if (_DbConnection.State == ConnectionState.Closed)
                _DbConnection.Open();
            DbDataReader _DbDataReader = null;
            try
            {
                _DbCommand.Parameters.Clear();
                _DbCommand.CommandText = aSQL.SQL;
                foreach (DbParameter aParam in aSQL.Parameters)
                    _DbCommand.Parameters.Add(aParam);
                _DbDataReader = _DbCommand.ExecuteReader();
                if (_DbDataReader.Read())
                {
                    int nFCount = _DbDataReader.FieldCount;
                    Dictionary<string, object> rowObj = new Dictionary<string, object>();
                    for (int i = 0; i < nFCount; i++)
                    {
                        string fName = _DbDataReader.GetName(i);
                        if (rowObj.ContainsKey(fName))
                        {
                            rowObj[fName] = _DbDataReader.GetValue(i);
                        }
                        else
                        {
                            rowObj.Add(fName, _DbDataReader.GetValue(i));
                        }
                    }
                    return rowObj;
                }
                _DbDataReader.Close();
                dbSQLs.Clear();

            }
            finally
            {
                if (_DbDataReader != null) _DbDataReader.Close();
                _DbConnection.Close();
            }
            return null;
        }
        public void Dispose()
        {
            try {
                int iSqlCmdCount = 0;
                foreach (ISQLEntity aSql in dbSQLs) {
                    if ((aSql.SqlType == SQLType.Insert) ||
                        (aSql.SqlType == SQLType.Update) ||
                        (aSql.SqlType == SQLType.Delete)) {
                            iSqlCmdCount++;
                            if (iSqlCmdCount > 2) { return; }
                    }
                }
                if(iSqlCmdCount>0)
                    Execute((iSqlCmdCount>2));
            }
            catch { }
        }
        int _IDbExecute.Execute(ISQLEntity aSQL)
        {
            return Execute(aSQL);
        }
        DataTable _IDbExecute.ExecuteQuery(ISQLEntity aSQL)
        {
            return ExecuteQuery(aSQL);
        }
        object _IDbExecute.ExecuteScalar(ISQLEntity aSQL)
        {
            return ExecuteScalar(aSQL);
        }
        Dictionary<string, object> _IDbExecute.ExecuteReader(ISQLEntity aSQL)
        {
            return ExecuteReader(aSQL);
        }
    }
   
    /// <summary>
    /// 提供数据查询的基础实现
    /// </summary>
    public abstract class DbQueryPrivder {
        protected DbMapperPrivder dbMapper;
        protected DataTable dbQryTable = null;
        protected ISQLEntity dbSQL = null;
        internal DbQueryPrivder(DbMapperPrivder aMapper,ISQLEntity aSql){
            dbMapper = aMapper;
            dbSQL = aSql;
        } 
        protected IEnumerator<T> GetEnumeratorCommon<T>() where T:new()
        {
            List<T> lstEnum = ToListCommon<T>();
            if (lstEnum != null) return lstEnum.GetEnumerator();
            return null;
        }
        protected List<T> ToListCommon<T>() where T : new()
        {
            DataTable aTable = ((_IDbExecute)dbMapper).ExecuteQuery(dbSQL);
            if (aTable != null)
            {
                List<T> lstEntitys = new List<T>();
                Type tType = typeof(T);
                TypeProxy tProxy = TypeProxy.Type2Proxy(tType);
                string sTableName = tType.Name;
                IDbTableNaming aTableMap = (new T() as IDbTableNaming);
                if (aTable != null) sTableName = aTableMap.DBTableName;
                foreach (DataRow aRow in aTable.Rows)
                {
                    T tIns = new T();
                    foreach (PropertyProxy aProperty in tProxy.DicPropertys.Values)
                    {
                        string sFieldName = aProperty.Property.Name;
                        if (!aTable.Columns.Contains(sFieldName))
                            sFieldName = string.Format("{0}{1}{2}", sTableName, DbMapper.CHAR_TABLE_PARAM_IDENTY, sFieldName);
                        if (aTable.Columns.Contains(sFieldName)){
                            if (aRow[sFieldName] != null)
                                EmitHelper.EmitSetter(aProperty.Setter, aProperty.Property, tIns, aRow[sFieldName]);
                             
                        }
                    }
                    lstEntitys.Add(tIns);
                }
                return lstEntitys;
            }
            return null;
        }
        protected IEnumerator GetEnumeratorCommon()
        {
            DataTable aTable = ((_IDbExecute)dbMapper).ExecuteQuery(dbSQL);
            return aTable.Rows.GetEnumerator();
        }
    }
    internal static class EmitHelper
    {
        internal delegate object GetterHandler(object source);
        internal delegate void SetterHandler(object source, object value);
        internal delegate object DefaultInstantiateObjectHandler();
        internal delegate object MethodHandler(object source, object[] parameters);
        internal static readonly MethodAttributes METHODATTR = MethodAttributes.Static | MethodAttributes.Public;
        internal static long nThunkNumber = 0;
        internal static DefaultInstantiateObjectHandler CreateNoParameterInstantiateHandler(Type type)
        {
            ConstructorInfo constructorInfo = type.GetConstructor(BindingFlags.Public |
               BindingFlags.NonPublic | BindingFlags.Instance, null, Type.EmptyTypes, null);
            if (constructorInfo == null) throw new ApplicationException("找不到相关构造");
            DynamicMethod dynamicMethod = new DynamicMethod(string.Format("EMIT_CONSTRUCT_{0}",Interlocked.Increment(ref nThunkNumber)), METHODATTR
                , CallingConventions.Standard, typeof(object), null, type, true);
            ILGenerator ilgen = dynamicMethod.GetILGenerator();
            ilgen.Emit(OpCodes.Newobj, constructorInfo);
            ilgen.Emit(OpCodes.Ret);
            return dynamicMethod.CreateDelegate(typeof(DefaultInstantiateObjectHandler)) as DefaultInstantiateObjectHandler;
        }
        internal static Delegate CreateInstantiateHandler(Type type, Type delegateType, params Type[] paramTypes)
        {
            if (paramTypes.Length == 0) throw new ApplicationException("构造参数错误");
            ConstructorInfo constructorInfo = type.GetConstructor(BindingFlags.Public |
               BindingFlags.NonPublic | BindingFlags.Instance, null, paramTypes, null);
            if (constructorInfo == null) throw new ApplicationException("找不到相关构造");

            DynamicMethod dynamicMethod = new DynamicMethod(string.Format("EMIT_CONSTRUCT_P_{0}", Interlocked.Increment(ref nThunkNumber)), METHODATTR,
                CallingConventions.Standard, typeof(object), paramTypes, type, true);
            ILGenerator ilgen = dynamicMethod.GetILGenerator();
            for (int i = 0, il = paramTypes.Length; i < il; i++)
            {
                switch (i)
                {
                    case 0: ilgen.Emit(OpCodes.Ldarg_0); break;
                    case 1: ilgen.Emit(OpCodes.Ldarg_1); break;
                    case 2: ilgen.Emit(OpCodes.Ldarg_2); break;
                    case 3: ilgen.Emit(OpCodes.Ldarg_3); break;
                    default: ilgen.Emit(OpCodes.Ldarg, i); break;
                }
            }
            ilgen.Emit(OpCodes.Newobj, constructorInfo);
            ilgen.Emit(OpCodes.Ret);
            Delegate del = dynamicMethod.CreateDelegate(delegateType);
            return del;
        }
        internal static GetterHandler CreateGetterHandler(Type type, PropertyInfo propertyInfo)
        {
            MethodInfo getterMethodInfo = propertyInfo.GetGetMethod(true);
            if (getterMethodInfo == null) { return null; }
            DynamicMethod dynamicGet = new DynamicMethod(string.Format("EMIT_GETMETHOD_{0}",Interlocked.Increment(ref nThunkNumber)),
                typeof(object), new Type[] { typeof(object) }, type, true);
            ILGenerator ilgen = dynamicGet.GetILGenerator();
            ilgen.Emit(OpCodes.Ldarg_0);
            ilgen.Emit(OpCodes.Call, getterMethodInfo);
            Boxing(getterMethodInfo.ReturnType, ilgen);
            ilgen.Emit(OpCodes.Ret);
            return dynamicGet.CreateDelegate(typeof(GetterHandler)) as GetterHandler;
        }
        internal static GetterHandler CreateGetterHandler(Type type, FieldInfo fieldInfo)
        {
            DynamicMethod dynamicGet = new DynamicMethod(string.Format("EMIT_GETFIELDVALUE_{0}", Interlocked.Increment(ref nThunkNumber)),
                typeof(object), new Type[] { typeof(object) }, type, true);
            ILGenerator ilgen = dynamicGet.GetILGenerator();
            ilgen.Emit(OpCodes.Ldarg_0);
            ilgen.Emit(OpCodes.Ldfld, fieldInfo);
            Boxing(fieldInfo.FieldType, ilgen);
            ilgen.Emit(OpCodes.Ret);
            return (GetterHandler)dynamicGet.CreateDelegate(typeof(GetterHandler));
        }
        internal static SetterHandler CreateSetterHandler(Type type, PropertyInfo propertyInfo)
        {
            MethodInfo setMethodInfo = propertyInfo.GetSetMethod(true);
            if (setMethodInfo == null) { return null; }
            DynamicMethod dynamicSet = new DynamicMethod(string.Format("EMIT_SETMETHOD_{0}",Interlocked.Increment(ref nThunkNumber)), typeof(void),
              new Type[] { typeof(object), typeof(object) }, type, true);
            Type propertyType = setMethodInfo.GetParameters()[0].ParameterType;
            ILGenerator ilgen = dynamicSet.GetILGenerator();
            ilgen.Emit(OpCodes.Ldarg_0);
            ilgen.Emit(OpCodes.Ldarg_1);
            UnBoxing(propertyType, ilgen);
            ilgen.Emit(OpCodes.Call, setMethodInfo);
            ilgen.Emit(OpCodes.Ret);
            return dynamicSet.CreateDelegate(typeof(SetterHandler)) as SetterHandler;
        }
        internal static SetterHandler CreateSetterHandler(Type type, FieldInfo fieldInfo)
        {
            DynamicMethod dynamicSet = new DynamicMethod(string.Format("EMIT_SETFIELDVALUE_{0}",Interlocked.Increment(ref nThunkNumber)), typeof(void),
               new Type[] { typeof(object), typeof(object) }, type, true);
            ILGenerator ilgen = dynamicSet.GetILGenerator();
            ilgen.Emit(OpCodes.Ldarg_0);
            ilgen.Emit(OpCodes.Ldarg_1);
            UnBoxing(fieldInfo.FieldType, ilgen);
            ilgen.Emit(OpCodes.Stfld, fieldInfo);
            ilgen.Emit(OpCodes.Ret);
            return (SetterHandler)dynamicSet.CreateDelegate(typeof(SetterHandler));
        }
        internal static MethodHandler CreateMethodHandler(Type type, MethodInfo methodInfo, int paramCount)
        {
            Type returnType = methodInfo.ReturnType;
            DynamicMethod dynamicMethod = new DynamicMethod(string.Format("EMIT_METHODINVOKE_P_{0}",Interlocked.Increment(ref nThunkNumber))
                , METHODATTR, CallingConventions.Standard,
               typeof(object), new Type[] { typeof(object), typeof(object[]) }, type, true);
            ILGenerator ilgen = dynamicMethod.GetILGenerator();
            if (!methodInfo.IsStatic) ilgen.Emit(OpCodes.Ldarg_0);
            for (int i = 0, il = paramCount; i < il; i++)
            {
                ilgen.Emit(OpCodes.Ldarg_1);
                ilgen.Emit(OpCodes.Ldc_I4, i);
                ilgen.Emit(OpCodes.Ldelem_Ref);
            }
            ilgen.Emit(OpCodes.Call, methodInfo);
            if (methodInfo.ReturnType.Name.ToLower() == "void")
                ilgen.Emit(OpCodes.Ldnull);
            else
                Boxing(methodInfo.ReturnType, ilgen);
            ilgen.Emit(OpCodes.Ret);
            return (dynamicMethod.CreateDelegate(typeof(MethodHandler)) as MethodHandler);
        }
        internal static Delegate CreateMethodHandler(Type type, MethodInfo methodInfo, Type delegateType)
        {
            Type returnType = methodInfo.ReturnType;
            ParameterInfo[] paramsInfo = methodInfo.GetParameters();
            Type[] paramType = null;
            if (methodInfo.IsStatic)
            {
                paramType = new Type[paramsInfo.Length];
                for (int i = 0, il = paramsInfo.Length; i < il; i++)
                {
                    ParameterInfo paramInfo = paramsInfo[i];
                    paramType[i] = paramInfo.ParameterType;
                }
            }
            else
            {
                paramType = new Type[paramsInfo.Length + 1];
                paramType[0] = typeof(object);
                for (int i = 0, il = paramsInfo.Length; i < il; i++)
                {
                    ParameterInfo paramInfo = paramsInfo[i];
                    paramType[i + 1] = paramInfo.ParameterType;
                }
            }
            DynamicMethod dynamicMethod = new DynamicMethod(string.Format("EMIT_METHODINVOKE_{0}",Interlocked.Increment(ref nThunkNumber))
                , METHODATTR, CallingConventions.Standard,
                (returnType == null) ? typeof(void) : returnType, (paramType.Length == 0) ? null : paramType, type, true);
            ILGenerator ilgen = dynamicMethod.GetILGenerator();
            for (int i = 0, il = paramType.Length; i < il; i++)
            {
                switch (i)
                {
                    case 0: ilgen.Emit(OpCodes.Ldarg_0); break;
                    case 1: ilgen.Emit(OpCodes.Ldarg_1); break;
                    case 2: ilgen.Emit(OpCodes.Ldarg_2); break;
                    case 3: ilgen.Emit(OpCodes.Ldarg_3); break;
                    default:
                        ilgen.Emit(OpCodes.Ldarg, i);
                        break;
                }
            }
            ilgen.Emit(OpCodes.Call, methodInfo);
            Boxing(methodInfo.ReturnType, ilgen);
            ilgen.Emit(OpCodes.Ret);
            return dynamicMethod.CreateDelegate(delegateType);
        }
        static void Boxing(Type type, ILGenerator ilgen)
        {
            if (type.IsValueType)
                ilgen.Emit(OpCodes.Box, type);
        }
        static void UnBoxing(Type type, ILGenerator ilgen)
        {
            if (type.IsValueType)
                ilgen.Emit(OpCodes.Unbox_Any, type);
        }
        internal static void EmitSetter(PropertyInfo Property, object Instance, object Value)
        {
            Type aType = Property.PropertyType;
            switch (aType.FullName)
            {
                case "System.Byte":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToByte(Value ?? 0)); break;
                case "System.SByte":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToSByte(Value ?? 0)); break;
                case "System.Int16":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToInt16(Value ?? 0)); break;
                case "System.UInt16":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToUInt16(Value ?? 0)); break;
                case "System.Int32":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToInt32(Value ?? 0)); break;
                case "System.UInt32":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToUInt32(Value ?? 0)); break;
                case "System.Int64":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToInt64(Value ?? 0)); break;
                case "System.UInt64":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToUInt64(Value ?? 0)); break;
                case "System.Single":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToSingle(Value ?? 0)); break;
                case "System.Double":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToDouble(Value ?? 0)); break;
                case "System.Decimal":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToDecimal(Value ?? 0)); break;
                case "System.Boolean":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Convert.ToBoolean(Value ?? 0)); break;
                case "System.Char":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Value); break;
                case "System.Guid":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance,
                        (Value != null) ? new Guid(Value.ToString()) : Guid.Empty); break;
                case "System.DateTime":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance,
                        (Value != null) ? Convert.ToDateTime(Value) : DateTime.MinValue); break;
                case "System.String":
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Value); break;
                default:
                    CreateSetterHandler(Property.PropertyType, Property)(Instance, Value); break;
            }
        }
        internal static void EmitSetter(SetterHandler handler,PropertyInfo Property, object Instance, object Value)
        {
            object obj = null;
            Type aType = Property.PropertyType;
            switch (aType.FullName)
            {
                case "System.Byte":
                    obj=Convert.ToByte(Value ?? 0); break;
                case "System.SByte":
                    obj=Convert.ToSByte(Value ?? 0); break;
                case "System.Int16":
                    obj=Convert.ToInt16(Value ?? 0); break;
                case "System.UInt16":
                    obj= Convert.ToUInt16(Value ?? 0); break;
                case "System.Int32":
                    obj=Convert.ToInt32(Value ?? 0); break;
                case "System.UInt32":
                    obj=Convert.ToUInt32(Value ?? 0); break;
                case "System.Int64":
                    obj=Convert.ToInt64(Value ?? 0); break;
                case "System.UInt64":
                    obj=Convert.ToUInt64(Value ?? 0); break;
                case "System.Single":
                    obj=Convert.ToSingle(Value ?? 0); break;
                case "System.Double":
                    obj=Convert.ToDouble(Value ?? 0); break;
                case "System.Decimal":
                    obj=Convert.ToDecimal(Value ?? 0); break;
                case "System.Boolean":
                    obj=Convert.ToBoolean(Value ?? 0); break;
                case "System.Char":
                    obj=Convert.ToChar(Value??'\0'); break;
                case "System.Guid":
                    obj=(Value != null) ? new Guid(Value.ToString()) : Guid.Empty; break;
                case "System.DateTime":
                    obj=(Value != null) ? Convert.ToDateTime(Value) : DateTime.MinValue; break;
                case "System.String":
                    obj=(Value??"").ToString(); break;
                default:
                    obj=Value; break;
            }
            handler(Instance, obj);
        }
    }
    internal class TypeProxy
    {
        private static Hashtable AllTypes = Hashtable.Synchronized(new Hashtable());
        public Type ClsType;
        internal EmitHelper.DefaultInstantiateObjectHandler DefaultConstruct;
        public Dictionary<string, PropertyProxy> DicPropertys;
        public TypeProxy(Type clType)
        {
            DicPropertys = new Dictionary<string, PropertyProxy>();
            ClsType = clType;
            if (clType.GetConstructor(Type.EmptyTypes)!=null) {
                DefaultConstruct = EmitHelper.CreateNoParameterInstantiateHandler(clType);
            }
            PropertyInfo[] arrProperty = clType.GetProperties(BindingFlags.Public|BindingFlags.Instance);
            foreach (PropertyInfo aPro in arrProperty)
            {
                EmitHelper.GetterHandler getter=null;
                if(aPro.CanRead) 
                   getter= EmitHelper.CreateGetterHandler(clType, aPro);
                EmitHelper.SetterHandler setter=null;
                if(aPro.CanWrite)
                   setter = EmitHelper.CreateSetterHandler(clType, aPro);
                DicPropertys.Add(aPro.Name,
                new PropertyProxy
                {
                    Property = aPro,
                    Getter = getter,
                    Setter = setter,
                    IsPrimaryKey = (aPro.GetCustomAttributes(PrimaryKeyAttribute.CLASS_TYPE, true).Length > 0)
                });
            }
        }
        public static TypeProxy Type2Proxy(Type clType)
        {
            TypeProxy proxy = AllTypes[clType] as TypeProxy;
            if (proxy == null)
            {
                lock (AllTypes)
                {
                    proxy = AllTypes[clType] as TypeProxy;
                    if (proxy == null)
                    {
                        proxy = new TypeProxy(clType);
                    }
                }
            }
            return proxy;
        }
    }
    public class PropertyProxy
    {
        public PropertyInfo Property;
        internal EmitHelper.GetterHandler Getter;
        internal EmitHelper.SetterHandler Setter;
        public bool IsPrimaryKey = false;
    }
    public class SqlBuilder {
        public delegate DbParameter AddOneParameterHandler1<T>(SQLType type, PropertyProxy property,
            object oValue, StringBuilder fields, StringBuilder parameters, StringBuilder wheres, T entity);

        public static ISQLEntity DefineCommandSQL<T,TSQLParam>(T entity, SQLType type, AddOneParameterHandler1<T> fnAddOneParameter)
            where TSQLParam : class 
        {
            IDbTableNaming aTable = entity as IDbTableNaming;
            Type tType = entity.GetType();
            TypeProxy PType = TypeProxy.Type2Proxy(tType);
            PropertyProxy[] arrPProxys = new PropertyProxy[PType.DicPropertys.Values.Count];
            PType.DicPropertys.Values.CopyTo(arrPProxys, 0);

            StringBuilder fields = new StringBuilder();
            StringBuilder parameters = new StringBuilder();
            StringBuilder wheres = new StringBuilder();
            PropertyProxy aProperty = arrPProxys[0];

            object oVal = aProperty.Getter(entity);
            List<TSQLParam> lstParameters = new List<TSQLParam>();
            if ((oVal != null) && !string.IsNullOrEmpty(oVal.ToString()))
            {
                lstParameters.Add(fnAddOneParameter(type, aProperty, oVal, fields, parameters, wheres, entity) as TSQLParam);
            }

            for (int i = 1, il = arrPProxys.Length; i < il; i++)
            {
                aProperty = arrPProxys[i];
                oVal = aProperty.Getter(entity);// EmitHelper.CreateGetterHandler(tType, aProperty)(entity);
                if ((oVal != null) && !string.IsNullOrEmpty(oVal.ToString()))
                {
                    lstParameters.Add(fnAddOneParameter(type, aProperty, oVal, fields, parameters, wheres, entity) as TSQLParam);
                }
            }
            if (type == SQLType.Insert)
            {
                if (fields.Length == 0) throw new Exception("error fields");
                if(typeof(TSQLParam)==typeof(OracleParameter)){
                  return new OracleSQLEntity(string.Format("INSERT INTO {0} ({1}) VALUES ({2})",
                    aTable.DBTableName, fields.ToString(), parameters.ToString()), SQLType.Insert, lstParameters.ToArray() as OracleParameter[]);
                }
                else if(typeof(TSQLParam)==typeof(SqlParameter)){
                    return new SqlServerSQLEntity(string.Format("INSERT INTO {0} ({1}) VALUES ({2})",
                    aTable.DBTableName, fields.ToString(), parameters.ToString()), SQLType.Insert, lstParameters.ToArray() as SqlParameter[]);
                }
                else{return null;}
            }
            else if (type == SQLType.Update)
            {
                if (fields.Length == 0) throw new Exception("error fields");
                if (wheres.Length == 0) throw new Exception("error where parameters,maybe can not set primary key");
                if (typeof(TSQLParam) == typeof(OracleParameter))
                {
                    return new OracleSQLEntity(string.Format("UPDATE {0} SET {1} WHERE {2}",
                        aTable.DBTableName, fields.ToString(), wheres.ToString()), SQLType.Update, lstParameters.ToArray() as OracleParameter[]);
                }
                else if (typeof(TSQLParam) == typeof(SqlParameter))
                {
                    return new SqlServerSQLEntity(string.Format("UPDATE {0} SET {1} WHERE {2}",
                        aTable.DBTableName, fields.ToString(), wheres.ToString()), SQLType.Update, lstParameters.ToArray() as SqlParameter[]);
                }
                else { return null; }
            }
            else if (type == SQLType.Delete)
            {
                if (wheres.Length == 0)
                    throw new Exception("error where parameters,maybe can not set primary key");
                if (typeof(TSQLParam) == typeof(OracleParameter))
                {
                    return new OracleSQLEntity(string.Format("DELETE FROM {0} WHERE {1}",
                        aTable.DBTableName, wheres.ToString()), SQLType.Delete, lstParameters.ToArray() as OracleParameter[]);
                }
                else if (typeof(TSQLParam) == typeof(SqlParameter))
                {
                    return new SqlServerSQLEntity(string.Format("DELETE FROM {0} WHERE {1}",
                        aTable.DBTableName, wheres.ToString()), SQLType.Delete, lstParameters.ToArray() as SqlParameter[]);
                }
                else { return null; }
            }
            else
            {
                throw new Exception("error sqltype");
            }
        }
        public delegate DbParameter AddOneDbParameterHandler2<T>(string name, ParameterDirection direct, Type entType, PropertyProxy aProperty,T entity);
        public static void DefineCommandSQL<T>(string sql, T entity, SQLType type, ISQLEntity aEntity, AddOneDbParameterHandler2<T> fnAddOneDbParameter)
        {
            Type tType = typeof(T);
            TypeProxy PType = TypeProxy.Type2Proxy(tType);
            string[] aArrs = sql.Split(DbMapper.SPLIT_SQL_PARAMS_IDENTY, StringSplitOptions.RemoveEmptyEntries);
            foreach (string strA in aArrs)
            {
                string strName = strA.Split(' ')[0];
                if (!string.IsNullOrEmpty(strName))
                {
                    PropertyProxy aProperty = PType.DicPropertys[strName];
                    if (aProperty != null)
                    {
                        aEntity.Parameters.Add(fnAddOneDbParameter(strName,ParameterDirection.Input,tType,aProperty,entity));
                    }
                }
            }
        }
        public delegate DbParameter AddOneDbParameterHandler3(string name, ParameterDirection direct, PropertyProxy aProperty, Type aType, object value);
        public static void DefineCommandSQL(ISQLEntity aEntity, AddOneDbParameterHandler3 fnAddOneDbParameter, string sql, SQLType type, params KeyValuePair<Type, Object>[] entitys)
        {
            string[] arrSQLParams = sql.Split(DbMapper.SPLIT_SQL_PARAMS_IDENTY, StringSplitOptions.RemoveEmptyEntries);
            Dictionary<Type, Object> kvParams = new Dictionary<Type, object>();
            foreach (KeyValuePair<Type, Object> kvPair in entitys) { kvParams.Add(kvPair.Key, kvPair.Value); }
            Dictionary<Type, Dictionary<string, PropertyProxy>> kvPropertys = new Dictionary<Type, Dictionary<string, PropertyProxy>>();
            Dictionary<Type, string> kvTableCls = new Dictionary<Type, string>();
            foreach (KeyValuePair<Type, Object> entity in entitys)
            {
                Type aType = entity.Key;
                TypeProxy aProxy = TypeProxy.Type2Proxy(aType);
                PropertyProxy[] arrProProxy = new PropertyProxy[aProxy.DicPropertys.Values.Count];
                aProxy.DicPropertys.Values.CopyTo(arrProProxy, 0);
                //PropertyInfo[] arrPropertys = aType.GetProperties(BindingFlags.Instance | BindingFlags.Public);
                IDbTableNaming aTable = null;
                if (entity.Value != null)
                {
                    aTable = entity.Value as IDbTableNaming;
                }
                else
                {
                    aTable = aProxy.DefaultConstruct() as IDbTableNaming;
                }
                if (aTable != null) { kvTableCls.Add(aType, aTable.DBTableName); }
                else { kvTableCls.Add(aType, aType.Name); }
                kvPropertys[aType] = new Dictionary<string, PropertyProxy>();
                foreach (PropertyProxy aProperty in arrProProxy)
                {
                    kvPropertys[aType].Add(aProperty.Property.Name, aProperty);
                }
            }
            for (int i = 1, il = arrSQLParams.Length; i < il; i++)
            {
                string sSQLParam = arrSQLParams[i];
                string sParamName = sSQLParam.Split(' ')[0];
                if (!string.IsNullOrEmpty(sParamName))
                {
                    string sParamTableName = sParamName;
                    string[] arrTableParamName = sParamTableName.Split(DbMapper.SPLIT_TABLE_PARAM_IDENTY, StringSplitOptions.RemoveEmptyEntries);
                    string sTableName = null;
                    if (arrTableParamName.Length == 2)
                    {
                        sTableName = (arrTableParamName[0] ?? "").Trim();
                        sParamName = arrTableParamName[1];
                    }
                    Type tarType = null;
                    if ((!string.IsNullOrEmpty(sTableName))
                        && (kvTableCls.ContainsValue(sTableName)))
                    {
                        foreach (Type dKey in kvTableCls.Keys)
                        {
                            if (kvTableCls[dKey] == sTableName)
                            {
                                tarType = dKey;
                                break;
                            }
                        }
                        PropertyProxy aProperty = kvPropertys[tarType][sParamName];
                        if (aProperty != null)
                        {
                            aEntity.Parameters.Add(fnAddOneDbParameter(sParamTableName
                                ,ParameterDirection.Input,aProperty,tarType,kvParams[tarType]));
                        }
                    }
                    else
                    {
                        foreach (Type aType in kvParams.Keys)
                        {
                            if (kvPropertys[aType].ContainsKey(sParamName))
                            {
                                PropertyProxy aProperty = kvPropertys[aType][sParamName];
                                if (aProperty != null)
                                {
                                    aEntity.Parameters.Add(fnAddOneDbParameter(sParamName
                                        ,ParameterDirection.Input,aProperty,aType,kvParams[aType]));
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    #region ORACLE
    /// <summary>
    /// Oracle实现
    /// </summary>
    public class OracleDbMapper : DbMapperPrivder, IDbMapper
    {
        
        private string _connString = "";
        private int _maxCmdTimeout = 300;
        static Dictionary<Type, OracleType> typeMap;
        static OracleDbMapper()
        {
            typeMap = new Dictionary<Type, OracleType>();
            typeMap[typeof(byte)] = OracleType.Number;
            typeMap[typeof(sbyte)] = OracleType.Number;
            typeMap[typeof(short)] = OracleType.Number;
            typeMap[typeof(ushort)] = OracleType.Number;
            typeMap[typeof(int)] = OracleType.Number;
            typeMap[typeof(uint)] = OracleType.Number;
            typeMap[typeof(long)] = OracleType.Number;
            typeMap[typeof(ulong)] = OracleType.Number;
            typeMap[typeof(float)] = OracleType.Number;
            typeMap[typeof(double)] = OracleType.Number;
            typeMap[typeof(decimal)] = OracleType.Number;
            typeMap[typeof(bool)] = OracleType.Number;
            typeMap[typeof(string)] = OracleType.VarChar;
            typeMap[typeof(char)] = OracleType.VarChar;
            typeMap[typeof(Guid)] = OracleType.VarChar;
            typeMap[typeof(DateTime)] = OracleType.DateTime;
            //typeMap[typeof(DateTimeOffset)] = DbType.DateTimeOffset;
            //typeMap[typeof(TimeSpan)] = DbType.Time;
            typeMap[typeof(byte[])] = OracleType.Blob;
            typeMap[typeof(byte?)] = OracleType.Number;
            typeMap[typeof(sbyte?)] = OracleType.Number;
            typeMap[typeof(short?)] = OracleType.Number;
            typeMap[typeof(ushort?)] = OracleType.Number;
            typeMap[typeof(int?)] = OracleType.Number;
            typeMap[typeof(uint?)] = OracleType.Number;
            typeMap[typeof(long?)] = OracleType.Number;
            typeMap[typeof(ulong?)] = OracleType.Number;
            typeMap[typeof(float?)] = OracleType.Number;
            typeMap[typeof(double?)] = OracleType.Number;
            typeMap[typeof(decimal?)] = OracleType.Number;
            typeMap[typeof(bool?)] = OracleType.Number;
            typeMap[typeof(char?)] = OracleType.VarChar;
            typeMap[typeof(Guid?)] = OracleType.VarChar;
            typeMap[typeof(DateTime?)] = OracleType.DateTime;
            //typeMap[typeof(DateTimeOffset?)] = DbType.DateTimeOffset;
            //typeMap[typeof(TimeSpan?)] = DbType.Time;
            typeMap[typeof(object)] = OracleType.Blob;
        }
        public OracleDbMapper() : base() { }
        public OracleDbMapper(int maxCmdTimeout)
            : this()
        {
            _maxCmdTimeout = maxCmdTimeout;
        }
        public override void Connection(string connString)
        {
            _connString = connString;
            InitConnection(DbMapper.PROVIDER_ORACLE);
            _DbConnection.ConnectionString = _connString;
            _DbCommand.CommandTimeout = _maxCmdTimeout;
        }
        private OracleParameter AddOneParameter<T>(SQLType type,PropertyProxy property,
            object oValue, StringBuilder fields, StringBuilder parameters, StringBuilder wheres, T entity)
        {
            Type tType = entity.GetType();
            object oVal = property.Getter(entity);
            if (type == SQLType.Insert)
            {
                if (fields.Length == 0)
                {
                    fields.Append(property.Property.Name);
                    parameters.Append(string.Format(":{0}", property.Property.Name));
                }
                else
                {
                    fields.Append(string.Format(",{0}", property.Property.Name));
                    parameters.Append(string.Format(",:{0}", property.Property.Name));
                }
            }
            else if (type == SQLType.Update)
            {
                if (property.IsPrimaryKey)
                {
                    if (wheres.Length == 0)
                    {
                        wheres.Append(string.Format("{0}=:{1}", property.Property.Name, property.Property.Name));
                    }
                    else
                    {
                        wheres.Append(string.Format("and {0}=:{1}", property.Property.Name, property.Property.Name));
                    }
                }
                else
                {
                    if (fields.Length == 0)
                    {
                        fields.Append(string.Format("{0}=:{1}", property.Property.Name, property.Property.Name));
                    }
                    else
                    {
                        fields.Append(string.Format(",{0}=:{1}", property.Property.Name, property.Property.Name));
                    }
                }
            }
            else if (type == SQLType.Delete)
            {
                if (wheres.Length == 0)
                {
                    wheres.Append(string.Format("{0}=:{1}", property.Property.Name, property.Property.Name));
                }
                else
                {
                    wheres.Append(string.Format("and {0}=:{1}", property.Property.Name, property.Property.Name));
                }
            }
            else { throw new Exception("error sqltype"); }
            OracleParameter aParameter = new OracleParameter();
            aParameter.ParameterName = property.Property.Name;
            aParameter.Direction = ParameterDirection.Input;
            aParameter.Value = oVal;
            aParameter.OracleType = typeMap[property.Property.PropertyType];
            return aParameter;
        }
        private OracleSQLEntity DefineCommandSQL<T>(string sql, T entity, SQLType type)
        {
            OracleSQLEntity aEntity = new OracleSQLEntity(sql, type);
            SqlBuilder.DefineCommandSQL(sql, entity, type, aEntity, AddOneDbParameter);
            return aEntity;
        }
        private DbParameter AddOneDbParameter<T>(string name, ParameterDirection direct,
            Type entType,PropertyProxy aProperty, T entity)
        {
            OracleParameter aParameter = new OracleParameter();
            aParameter.ParameterName = name;
            aParameter.Direction = ParameterDirection.Input;
            aParameter.OracleType = typeMap[aProperty.Property.PropertyType];
            aParameter.Value =aProperty.Getter(entity);
            return aParameter;
        }
        private OracleSQLEntity DefineCommandSQL(string sql, SQLType type, params KeyValuePair<Type, Object>[] entitys)
        {
            OracleSQLEntity aEntity = new OracleSQLEntity(sql, type);
            SqlBuilder.DefineCommandSQL(aEntity, AddOneDbParameter, sql, type, entitys);
            return aEntity;
        }
        private DbParameter AddOneDbParameter(string name, ParameterDirection direct,PropertyProxy aProperty, Type aType, object value)
        {
            OracleParameter aParameter = new OracleParameter();
            aParameter.ParameterName = name;
            aParameter.Direction = direct;
            aParameter.OracleType = typeMap[aProperty.Property.PropertyType];
            aParameter.Value = aProperty.Getter(value);
            return aParameter;
        }
        public void Insert<T>(T entity)
        {
            dbSQLs.Add(SqlBuilder.DefineCommandSQL<T,OracleParameter>(entity, SQLType.Insert,AddOneParameter));
        }
        public void Update<T>(T entity)
        {
            dbSQLs.Add(SqlBuilder.DefineCommandSQL<T, OracleParameter>(entity, SQLType.Update, AddOneParameter));
        }
        public void Delete<T>(T entity)
        {
            dbSQLs.Add(SqlBuilder.DefineCommandSQL<T, OracleParameter>(entity, SQLType.Delete, AddOneParameter));
        }
        public void Action<T>(string sql, T entity)
        {
            SQLType type = SQLType.Insert;
            string tSql = sql.TrimStart(' ').ToUpper();
            if (tSql.StartsWith("INSERT")) { type = SQLType.Insert; }
            else if (tSql.StartsWith("UPDATE")) { type = SQLType.Update; }
            else if (tSql.StartsWith("DELETE")) { type = SQLType.Delete; }
            dbSQLs.Add(DefineCommandSQL<T>(sql, entity, type));
        }
        public void Action(string sql, IDbDynParameters parameters)
        {
            OracleDynParameters oraParameters = parameters as OracleDynParameters;
            string tSql = sql.TrimStart(' ').ToUpper();
            SQLType type = SQLType.StoreProc;
            if (oraParameters == null)
                throw new ApplicationException("arguments type cast exception");
            OracleParameter[] arrParameters = new OracleParameter[oraParameters.Count];
            oraParameters.Values.CopyTo(arrParameters, 0);
            if (tSql.StartsWith("@"))
            {
                tSql = tSql.Substring(1, tSql.Length - 1);
                type = SQLType.StoreProc;
            }
            else
            {
                if (tSql.StartsWith("INSERT")) { type = SQLType.Insert; }
                else if (tSql.StartsWith("UPDATE")) { type = SQLType.Update; }
                else if (tSql.StartsWith("DELETE")) { type = SQLType.Delete; }
            }
            dbSQLs.Add(new OracleSQLEntity(tSql, type, arrParameters));
        }
        #if NET4_0
        public IDbQuery<T> Query<T>(string sql, dynamic entity)
            where T : class,new()
        {

            Type dynType = (entity != null) ? entity.GetType() : typeof(object);
            return new OracleDbQuery<T>(this, DefineCommandSQL(sql, SQLType.Select
                , new KeyValuePair<Type, Object>(dynType, entity))
                , new Type[] { typeof(T) });
        }
        #endif
        public IDbQuery<T> Query<T>(string sql, T entity)
            where T : class,new()
        {
            return new OracleDbQuery<T>(this
                , DefineCommandSQL(sql, SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity))
                , new Type[] { typeof(T) });
        }
        public IDbQuery<T> Query<T, T1>(string sql, T entity, T1 entity1)
            where T : class, new()
            where T1 : class, new()
        {
            return new OracleDbQuery<T>(this
            , DefineCommandSQL(sql, SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1))
            , new Type[] { typeof(T), typeof(T1) });
        }
        public IDbQuery<T> Query<T, T1, T2>(string sql, T entity, T1 entity1, T2 entity2)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
        {
            return new OracleDbQuery<T>(this
            , DefineCommandSQL(sql, SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1)
            , new KeyValuePair<Type, Object>(typeof(T2), entity2))
            , new Type[] { typeof(T), typeof(T1), typeof(T2) });
        }
        public IDbQuery<T> Query<T, T1, T2, T3>(string sql, T entity, T1 entity1, T2 entity2, T3 entity3)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new()
        {
            return new OracleDbQuery<T>(this
            , DefineCommandSQL(sql, SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1)
            , new KeyValuePair<Type, Object>(typeof(T2), entity2)
            , new KeyValuePair<Type, Object>(typeof(T3), entity3)),
            new Type[] { typeof(T), typeof(T1), typeof(T2), typeof(T3) });
        }
        public IDbQuery<T> Query<T, T1, T2, T3, T4>(string sql, T entity, T1 entity1, T2 entity2, T3 entity3, T4 entity4)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new()
            where T4 : class, new()
        {
            return new OracleDbQuery<T>(this, DefineCommandSQL(sql
            , SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1)
            , new KeyValuePair<Type, Object>(typeof(T2), entity2)
            , new KeyValuePair<Type, Object>(typeof(T3), entity3)
            , new KeyValuePair<Type, Object>(typeof(T4), entity4)),
            new Type[] { typeof(T), typeof(T1), typeof(T2), typeof(T3), typeof(T4) });
        }
        public IDbQuery<T> Query<T, T1, T2, T3, T4, T5>(string sql, T entity, T1 entity1, T2 entity2, T3 entity3, T4 entity4, T5 entity5)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new()
            where T4 : class, new()
            where T5 : class, new()
        {
            return new OracleDbQuery<T>(this, DefineCommandSQL(sql
            , SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1)
            , new KeyValuePair<Type, Object>(typeof(T2), entity2)
            , new KeyValuePair<Type, Object>(typeof(T3), entity3)
            , new KeyValuePair<Type, Object>(typeof(T4), entity4)
            , new KeyValuePair<Type, Object>(typeof(T5), entity5)),
            new Type[] { typeof(T), typeof(T1), typeof(T2), typeof(T3), typeof(T4), typeof(T5) });
        }
        public IDbQuery<T> Query<T, T1, T2, T3, T4, T5, T6>(string sql, T entity, T1 entity1, T2 entity2, T3 entity3, T4 entity4, T5 entity5, T6 entity6)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new()
            where T4 : class, new()
            where T5 : class, new()
            where T6 : class, new()
        {
            return new OracleDbQuery<T>(this, DefineCommandSQL(sql, SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1)
            , new KeyValuePair<Type, Object>(typeof(T2), entity2)
            , new KeyValuePair<Type, Object>(typeof(T3), entity3)
            , new KeyValuePair<Type, Object>(typeof(T4), entity4)
            , new KeyValuePair<Type, Object>(typeof(T5), entity5)
            , new KeyValuePair<Type, Object>(typeof(T6), entity6)),
            new Type[] { typeof(T), typeof(T1), typeof(T2), typeof(T3), typeof(T4), typeof(T5), typeof(T6) });
        }
    }
    public class OracleDbQuery<T> : DbQueryPrivder, IDbQuery<T> where T : new()
    {
        private static int _Identity = 0;
        private List<Type> lstInlineTypes;
        internal OracleDbQuery(OracleDbMapper aMapper, OracleSQLEntity aSql) :
            base(aMapper, aSql) { }
        internal OracleDbQuery(OracleDbMapper aMapper, OracleSQLEntity aSql, Type[] inlineTypes) :
            base(aMapper, aSql)
        {
            lstInlineTypes = new List<Type>();
            lstInlineTypes.AddRange(inlineTypes);
        }
        public T First
        {
            get
            {
                Dictionary<string, object> aReader = ((_IDbExecute)dbMapper).ExecuteReader(dbSQL);
                Type tType = typeof(T);
                TypeProxy tProxy = TypeProxy.Type2Proxy(tType); 
                T tIns = new T();
                foreach (PropertyProxy aProperty in tProxy.DicPropertys.Values)
                {
                    if (aReader.ContainsKey(aProperty.Property.Name))
                    {
                        if (aReader[aProperty.Property.Name] != null)
                            EmitHelper.EmitSetter(aProperty.Setter, aProperty.Property, tIns, aReader[aProperty.Property.Name]);
                    }
                }
                return tIns;
            }
        }
        public List<TReturn> ToList<TReturn>() where TReturn : new()
        {
            if ((typeof(TReturn) != typeof(T)) &&
                ((lstInlineTypes == null) || (!lstInlineTypes.Contains(typeof(TReturn)))))
                throw new InvalidCastException("目标类型不包括在查询类型中");
            return ToListCommon<TReturn>();
        }
        public IEnumerator<TReturn> GetEnumerator<TReturn>() where TReturn : new()
        {
            if ((typeof(TReturn) != typeof(T)) &&
                ((lstInlineTypes == null) || (!lstInlineTypes.Contains(typeof(TReturn)))))
                throw new InvalidCastException("目标类型不包括在查询类型中");
            return GetEnumeratorCommon<TReturn>();
        }
        public IEnumerator<T> GetEnumerator()
        {
            return GetEnumeratorCommon<T>();
        }
        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumeratorCommon();
        }
        public List<T> ToList()
        {
            return ToListCommon<T>();
        }
        public int Count
        {
            get
            {
                OracleDbMapper oraDbMapper = (OracleDbMapper)dbMapper;
                OracleParameter[] parameters = new OracleParameter[dbSQL.Parameters.Count];
                dbSQL.Parameters.CopyTo(parameters, 0);
                ISQLEntity oraSQL = new OracleSQLEntity(string.Format("SELECT COUNT(*) FROM ({0})", dbSQL.SQL)
                    , SQLType.Select, parameters);
                string oCount = (((_IDbExecute)oraDbMapper).ExecuteScalar(oraSQL) ?? -1).ToString();
                int nCount = -1;
                int.TryParse(oCount, out nCount);
                return nCount;
            }
        }
        public DataTable ToTable()
        {
            return ((_IDbExecute)dbMapper).ExecuteQuery(dbSQL);
        }
        private int Identity
        {
            get
            {
                int iden = Interlocked.Increment(ref _Identity);
                return (iden < 999999999) ? iden : Interlocked.Exchange(ref _Identity, 1);
            }
        }
        public IDbQuery<T> Skip(int index)
        {
            OracleDbMapper oraDbMapper = (OracleDbMapper)dbMapper;
            OracleParameter[] parameters = new OracleParameter[dbSQL.Parameters.Count];
            dbSQL.Parameters.CopyTo(parameters, 0);
            OracleSQLEntity oraSQL = new OracleSQLEntity(
                string.Format("SELECT * FROM (SELECT T.*,ROWNUM as R##{2} FROM ({0}) T) WHERE R##{2}>{1}"
                , dbSQL.SQL, index, Identity)
                , SQLType.Select, parameters);

            return (lstInlineTypes == null)
                ? new OracleDbQuery<T>(oraDbMapper, oraSQL)
                : new OracleDbQuery<T>(oraDbMapper, oraSQL, lstInlineTypes.ToArray());
        }
        public IDbQuery<T> Take(int count)
        {
            OracleDbMapper oraDbMapper = (OracleDbMapper)dbMapper;
            OracleParameter[] parameters = new OracleParameter[dbSQL.Parameters.Count];
            dbSQL.Parameters.CopyTo(parameters, 0);
            OracleSQLEntity oraSQL = new OracleSQLEntity(
                string.Format("SELECT * FROM (SELECT T.*,ROWNUM as R##{2} FROM ({0}) T) WHERE R##{2}<={1}"
                , dbSQL.SQL, count, Identity)
                , SQLType.Select, parameters);
            return (lstInlineTypes == null)
                ? new OracleDbQuery<T>(oraDbMapper, oraSQL)
                : new OracleDbQuery<T>(oraDbMapper, oraSQL, lstInlineTypes.ToArray());
        }
        public IDbQuery<T> OrderBy(string name, bool direct)
        {
            OracleDbMapper oraDbMapper = (OracleDbMapper)dbMapper;
            OracleParameter[] parameters = new OracleParameter[dbSQL.Parameters.Count];
            dbSQL.Parameters.CopyTo(parameters, 0);
            OracleSQLEntity oraSQL = new OracleSQLEntity(
                string.Format("SELECT * FROM ({0}) ORDER BY {1} {2}"
                , dbSQL.SQL,name,direct?"ASC":"DESC")
                , SQLType.Select, parameters);

            return (lstInlineTypes == null)
                ? new OracleDbQuery<T>(oraDbMapper, oraSQL)
                : new OracleDbQuery<T>(oraDbMapper, oraSQL, lstInlineTypes.ToArray());
        }
    }
    internal class OracleSQLEntity : ISQLEntity
    {
        private string _SQL = null;
        private SQLType _OperType;
        private List<DbParameter> _Parameters = null;
        public string SQL
        {
            get { return _SQL; }
        }
        public List<DbParameter> Parameters
        {
            get { return _Parameters; }
        }
        public SQLType SqlType
        {
            get { return _OperType; }
        }
        public OracleSQLEntity(string sql, SQLType type)
        {
            _SQL = sql;
            _OperType = type;
            _Parameters = new List<DbParameter>();
        }
        public OracleSQLEntity(string sql, SQLType type, OracleParameter[] parameters)
            : this(sql, type)
        {
            if (parameters.Length > 0)
                _Parameters.AddRange(parameters);
        }
    }
    public class OracleDynParameters : Dictionary<string, OracleParameter>, IDbDynParameters
    {
        public T Get<T>(string name)
        {
            if (ContainsKey(name))
            {
                return (T)this[name].Value;
            }
            throw new ApplicationException("name in parameters is not exists");
        }
        public void Add<T>(string name, T value)
        {
            Add<T>(name, value, ParameterDirection.Input);
        }
        public void Add<T>(string name, T value, ParameterDirection direction)
        {
            Add<T>(name, value, direction, -1);
        }
        public void Add<T>(string name, T value, ParameterDirection direction, int size)
        {
            if (ContainsKey(name))
                throw new ApplicationException("name is contain");
            OracleParameter aParameter = new OracleParameter();
            aParameter.Direction = direction;
            aParameter.ParameterName = name;
            if (size > 0) aParameter.Size = size;
            aParameter.Value = value;
            base.Add(name, aParameter);
        }
    }
    #endregion

    #region SQLSERVER
    /// <summary>
    /// MSSQLServer实现
    /// </summary>
    public class SqlServerDbMapper : DbMapperPrivder, IDbMapper {
        static Dictionary<Type, SqlDbType> typeMap;
        static SqlServerDbMapper() {
            typeMap = new Dictionary<Type, SqlDbType>();
            typeMap[typeof(byte)] =SqlDbType.TinyInt;
            typeMap[typeof(sbyte)] = SqlDbType.TinyInt;
            typeMap[typeof(short)] = SqlDbType.SmallInt;
            typeMap[typeof(ushort)] = SqlDbType.SmallInt;
            typeMap[typeof(int)] = SqlDbType.Int;
            typeMap[typeof(uint)] = SqlDbType.Int;
            typeMap[typeof(long)] = SqlDbType.BigInt;
            typeMap[typeof(ulong)] = SqlDbType.BigInt;
            typeMap[typeof(float)] = SqlDbType.Float;
            typeMap[typeof(double)] = SqlDbType.Real;
            typeMap[typeof(decimal)] = SqlDbType.Decimal;
            typeMap[typeof(bool)] = SqlDbType.Bit;
            typeMap[typeof(string)] = SqlDbType.VarChar;
            typeMap[typeof(char)] = SqlDbType.TinyInt;
            typeMap[typeof(Guid)] =SqlDbType.UniqueIdentifier;
            typeMap[typeof(DateTime)] = SqlDbType.DateTime;
            //typeMap[typeof(DateTimeOffset)] = DbType.DateTimeOffset;
            //typeMap[typeof(TimeSpan)] = DbType.Time;
            typeMap[typeof(byte[])] = SqlDbType.Binary;
            typeMap[typeof(byte?)] = SqlDbType.TinyInt;
            typeMap[typeof(sbyte?)] = SqlDbType.TinyInt;
            typeMap[typeof(short?)] = SqlDbType.SmallInt;
            typeMap[typeof(ushort?)] = SqlDbType.SmallInt;
            typeMap[typeof(int?)] = SqlDbType.Int;
            typeMap[typeof(uint?)] = SqlDbType.Int;
            typeMap[typeof(long?)] = SqlDbType.BigInt;
            typeMap[typeof(ulong?)] = SqlDbType.BigInt;
            typeMap[typeof(float?)] = SqlDbType.Float;
            typeMap[typeof(double?)] = SqlDbType.Real;
            typeMap[typeof(decimal?)] = SqlDbType.Decimal;
            typeMap[typeof(bool?)] = SqlDbType.Bit;
            typeMap[typeof(char?)] = SqlDbType.TinyInt;
            typeMap[typeof(Guid?)] = SqlDbType.UniqueIdentifier;
            typeMap[typeof(DateTime?)] = SqlDbType.DateTime;
            //typeMap[typeof(DateTimeOffset?)] = DbType.DateTimeOffset;
            //typeMap[typeof(TimeSpan?)] = DbType.Time;
            typeMap[typeof(object)] = SqlDbType.Variant;
        }
        private SqlParameter AddOneParameter<T>(SQLType type,PropertyProxy property,
                     object oValue, StringBuilder fields, StringBuilder parameters, StringBuilder wheres, T entity)
        {
            Type tType = entity.GetType();
            object oVal = property.Getter(entity);
            if (type == SQLType.Insert)
            {
                if (fields.Length == 0)
                {
                    fields.Append(property.Property.Name);
                    parameters.Append(string.Format("@{0}", property.Property.Name));
                }
                else
                {
                    fields.Append(string.Format(",{0}", property.Property.Name));
                    parameters.Append(string.Format(",@{0}", property.Property.Name));
                }
            }
            else if (type == SQLType.Update)
            {
                if (property.IsPrimaryKey)
                {
                    if (wheres.Length == 0)
                    {
                        wheres.Append(string.Format("{0}=@{1}", property.Property.Name, property.Property.Name));
                    }
                    else
                    {
                        wheres.Append(string.Format("and {0}=@{1}", property.Property.Name, property.Property.Name));
                    }
                }
                else
                {
                    if (fields.Length == 0)
                    {
                        fields.Append(string.Format("{0}=@{1}", property.Property.Name, property.Property.Name));
                    }
                    else
                    {
                        fields.Append(string.Format(",{0}=@{1}", property.Property.Name, property.Property.Name));
                    }
                }
            }
            else if (type == SQLType.Delete)
            {
                if (wheres.Length == 0)
                {
                    wheres.Append(string.Format("{0}=@{1}", property.Property.Name, property.Property.Name));
                }
                else
                {
                    wheres.Append(string.Format("and {0}=@{1}", property.Property.Name, property.Property.Name));
                }
            }
            else { throw new Exception("error sqltype"); }
            SqlParameter aParameter = new SqlParameter();
            aParameter.ParameterName = property.Property.Name;
            aParameter.Direction = ParameterDirection.Input;
            aParameter.SqlValue = oVal;
            aParameter.SqlDbType = typeMap[property.Property.PropertyType];
            return aParameter;
        }
        private SqlServerSQLEntity DefineCommandSQL(string sql, SQLType type, params KeyValuePair<Type, Object>[] entitys)
        {
            SqlServerSQLEntity aEntity = new SqlServerSQLEntity(sql, type);
            SqlBuilder.DefineCommandSQL(aEntity, AddOneDbParameter, sql, type, entitys);
            return aEntity;
        }
        private SqlServerSQLEntity DefineCommandSQL<T>(string sql, T entity, SQLType type)
        {
            SqlServerSQLEntity aEntity = new SqlServerSQLEntity(sql, type);
            SqlBuilder.DefineCommandSQL(sql, entity, type, aEntity, AddOneDbParameter);
            return aEntity;
        }
        private DbParameter AddOneDbParameter(string name, ParameterDirection direct, PropertyProxy aProperty, Type aType, object value)
        {
            SqlParameter aParameter = new SqlParameter();
            aParameter.ParameterName = name;
            aParameter.Direction = direct;
            aParameter.SqlDbType = typeMap[aProperty.Property.PropertyType];
            aParameter.SqlValue =aProperty.Getter(value);
            return aParameter;
        }
        private DbParameter AddOneDbParameter<T>(string name, ParameterDirection direct,
            Type entType,PropertyProxy aProperty, T entity)
        {
            SqlParameter aParameter = new SqlParameter();
            aParameter.ParameterName = name;
            aParameter.Direction = ParameterDirection.Input;
            aParameter.SqlDbType = typeMap[aProperty.Property.PropertyType];
            aParameter.SqlValue =aProperty.Getter(entity); 
            return aParameter;
        }
        #if NET4_0
        public IDbQuery<T> Query<T>(string sql, dynamic entity) where T : class, new()
        {
            Type dynType = (entity != null) ? entity.GetType() : typeof(object);
            return new SqlServerDbQuery<T>(this, DefineCommandSQL(sql, SQLType.Select
                , new KeyValuePair<Type, Object>(dynType, entity))
                , new Type[] { typeof(T) });
        }
        #endif
        public override void Connection(string connString)
        {
            InitConnection(DbMapper.PROVIDER_SQLSERVER);
            _DbConnection.ConnectionString = connString;
            _DbCommand.CommandTimeout = 30000;
        }
        public IDbQuery<T> Query<T>(string sql, T entity) where T : class, new()
        {
            return new SqlServerDbQuery<T>(this
                , DefineCommandSQL(sql, SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity))
                , new Type[] { typeof(T) });
        }
        public IDbQuery<T> Query<T, T1>(string sql, T entity, T1 entity1)
            where T : class, new()
            where T1 : class, new()
        {
            return new SqlServerDbQuery<T>(this
            , DefineCommandSQL(sql, SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1))
            , new Type[] { typeof(T), typeof(T1) });
        }
        public IDbQuery<T> Query<T, T1, T2>(string sql, T entity, T1 entity1, T2 entity2)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
        {
            return new SqlServerDbQuery<T>(this
            , DefineCommandSQL(sql, SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1)
            , new KeyValuePair<Type, Object>(typeof(T2), entity2))
            , new Type[] { typeof(T), typeof(T1), typeof(T2) });
        }
        public IDbQuery<T> Query<T, T1, T2, T3>(string sql, T entity, T1 entity1, T2 entity2, T3 entity3)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new()
        {
            return new SqlServerDbQuery<T>(this
            , DefineCommandSQL(sql, SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1)
            , new KeyValuePair<Type, Object>(typeof(T2), entity2)
            , new KeyValuePair<Type, Object>(typeof(T3), entity3)),
            new Type[] { typeof(T), typeof(T1), typeof(T2), typeof(T3) });
        }
        public IDbQuery<T> Query<T, T1, T2, T3, T4>(string sql, T entity, T1 entity1, T2 entity2, T3 entity3, T4 entity4)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new()
            where T4 : class, new()
        {
            return new SqlServerDbQuery<T>(this, DefineCommandSQL(sql
            , SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1)
            , new KeyValuePair<Type, Object>(typeof(T2), entity2)
            , new KeyValuePair<Type, Object>(typeof(T3), entity3)
            , new KeyValuePair<Type, Object>(typeof(T4), entity4)),
            new Type[] { typeof(T), typeof(T1), typeof(T2), typeof(T3), typeof(T4) });
        }
        public IDbQuery<T> Query<T, T1, T2, T3, T4, T5>(string sql, T entity, T1 entity1, T2 entity2, T3 entity3, T4 entity4, T5 entity5)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new()
            where T4 : class, new()
            where T5 : class, new()
        {
            return new SqlServerDbQuery<T>(this, DefineCommandSQL(sql
            , SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1)
            , new KeyValuePair<Type, Object>(typeof(T2), entity2)
            , new KeyValuePair<Type, Object>(typeof(T3), entity3)
            , new KeyValuePair<Type, Object>(typeof(T4), entity4)
            , new KeyValuePair<Type, Object>(typeof(T5), entity5)),
            new Type[] { typeof(T), typeof(T1), typeof(T2), typeof(T3), typeof(T4), typeof(T5) });
        }
        public IDbQuery<T> Query<T, T1, T2, T3, T4, T5, T6>(string sql, T entity, T1 entity1, T2 entity2, T3 entity3, T4 entity4, T5 entity5, T6 entity6)
            where T : class, new()
            where T1 : class, new()
            where T2 : class, new()
            where T3 : class, new()
            where T4 : class, new()
            where T5 : class, new()
            where T6 : class, new()
        {
            return new SqlServerDbQuery<T>(this, DefineCommandSQL(sql, SQLType.Select, new KeyValuePair<Type, Object>(typeof(T), entity)
            , new KeyValuePair<Type, Object>(typeof(T1), entity1)
            , new KeyValuePair<Type, Object>(typeof(T2), entity2)
            , new KeyValuePair<Type, Object>(typeof(T3), entity3)
            , new KeyValuePair<Type, Object>(typeof(T4), entity4)
            , new KeyValuePair<Type, Object>(typeof(T5), entity5)
            , new KeyValuePair<Type, Object>(typeof(T6), entity6)),
            new Type[] { typeof(T), typeof(T1), typeof(T2), typeof(T3), typeof(T4), typeof(T5), typeof(T6) });
        }
        public void Action<T>(string sql, T entity)
        {
            SQLType type = SQLType.Insert;
            string tSql = sql.TrimStart(' ').ToUpper();
            if (tSql.StartsWith("INSERT")) { type = SQLType.Insert; }
            else if (tSql.StartsWith("UPDATE")) { type = SQLType.Update; }
            else if (tSql.StartsWith("DELETE")) { type = SQLType.Delete; }
            dbSQLs.Add(DefineCommandSQL<T>(sql, entity, type));
        }
        public void Action(string sql, IDbDynParameters parameters)
        {
            SqlServerDynParameters oraParameters = parameters as SqlServerDynParameters;
            string tSql = sql.TrimStart(' ').ToUpper();
            SQLType type = SQLType.StoreProc;
            if (oraParameters == null)
                throw new ApplicationException("arguments type cast exception");
            SqlParameter[] arrParameters = new SqlParameter[oraParameters.Count];
            oraParameters.Values.CopyTo(arrParameters, 0);
            if (tSql.StartsWith("@"))
            {
                tSql = tSql.Substring(1, tSql.Length - 1);
                type = SQLType.StoreProc;
            }
            else
            {
                if (tSql.StartsWith("INSERT")) { type = SQLType.Insert; }
                else if (tSql.StartsWith("UPDATE")) { type = SQLType.Update; }
                else if (tSql.StartsWith("DELETE")) { type = SQLType.Delete; }
            }
            dbSQLs.Add(new SqlServerSQLEntity(tSql, type, arrParameters));
        }
        public void Insert<T>(T entity)
        {
            dbSQLs.Add(SqlBuilder.DefineCommandSQL<T,SqlParameter>(entity, SQLType.Insert, AddOneParameter));
        }
        public void Update<T>(T entity)
        {
            dbSQLs.Add(SqlBuilder.DefineCommandSQL<T, SqlParameter>(entity, SQLType.Update, AddOneParameter));
        }
        public void Delete<T>(T entity)
        {
            dbSQLs.Add(SqlBuilder.DefineCommandSQL<T, SqlParameter>(entity, SQLType.Delete, AddOneParameter));
        }
    }
    public class SqlServerDbQuery<T> : DbQueryPrivder, IDbQuery<T> where T : new()
    {
        private static int _Identity = 0;
        private List<Type> lstInlineTypes;
        private string _SortField;
        public string SortField { get { return _SortField; } }
        private string _SortOrder;
        public string SortOrder { get { return _SortOrder; } }
        internal SqlServerDbQuery(SqlServerDbMapper aMapper, SqlServerSQLEntity aSql) :
            base(aMapper, aSql) { }
        internal SqlServerDbQuery(SqlServerDbMapper aMapper, SqlServerSQLEntity aSql, Type[] inlineTypes) :
            base(aMapper, aSql)
        {
            lstInlineTypes = new List<Type>();
            lstInlineTypes.AddRange(inlineTypes);
        }
        public T First
        {
            get
            {
                SortedInject();
                Dictionary<string, object> aReader = ((_IDbExecute)dbMapper).ExecuteReader(dbSQL);
                Type tType = typeof(T);
                TypeProxy tProxy = TypeProxy.Type2Proxy(tType);
                T tIns = new T();
                foreach (PropertyProxy aProperty in tProxy.DicPropertys.Values)
                {
                    if (aReader.ContainsKey(aProperty.Property.Name))
                    {
                        if (aReader[aProperty.Property.Name] != null)
                            EmitHelper.EmitSetter(aProperty.Setter, aProperty.Property, tIns, aReader[aProperty.Property.Name]);
                            
                    }
                }
                return tIns;
            }
        }
        public List<TReturn> ToList<TReturn>() where TReturn : new()
        {
            if ((typeof(TReturn) != typeof(T)) &&
                ((lstInlineTypes == null) || (!lstInlineTypes.Contains(typeof(TReturn)))))
                throw new InvalidCastException("目标类型不包括在查询类型中");
            SortedInject();
            return ToListCommon<TReturn>();
        }
        public IEnumerator<TReturn> GetEnumerator<TReturn>() where TReturn : new()
        {
            if ((typeof(TReturn) != typeof(T)) &&
                ((lstInlineTypes == null) || (!lstInlineTypes.Contains(typeof(TReturn)))))
                throw new InvalidCastException("目标类型不包括在查询类型中");
            SortedInject();
            return GetEnumeratorCommon<TReturn>();
        }
        public IEnumerator<T> GetEnumerator()
        {
            SortedInject();
            return GetEnumeratorCommon<T>();
        }
        IEnumerator IEnumerable.GetEnumerator()
        {
            SortedInject();
            return GetEnumeratorCommon();
        }
        public List<T> ToList()
        {
            SortedInject();
            return ToListCommon<T>();
        }
        public int Count
        {
            get
            {
                SqlServerDbMapper oraDbMapper = (SqlServerDbMapper)dbMapper;
                SqlParameter[] parameters = new SqlParameter[dbSQL.Parameters.Count];
                dbSQL.Parameters.CopyTo(parameters, 0);
                ISQLEntity oraSQL = new SqlServerSQLEntity(string.Format("SELECT COUNT(*) FROM ({0}) TABLE_COUNT", dbSQL.SQL)
                    , SQLType.Select, parameters);
                string oCount = (((_IDbExecute)oraDbMapper).ExecuteScalar(oraSQL) ?? -1).ToString();
                int nCount = -1;
                int.TryParse(oCount, out nCount);
                return nCount;
            }
        }
        public DataTable ToTable()
        {
            SortedInject();
            return ((_IDbExecute)dbMapper).ExecuteQuery(dbSQL);
        }
        protected void SortedInject()
        {
            if (!string.IsNullOrEmpty(_SortField))
            {
                dbSQL = new SqlServerSQLEntity(string.Format("SELECT * from ({0}) TABLE_FIRST_1 order by {1} {2}"
                        , dbSQL.SQL, _SortField, _SortOrder), SQLType.Select);
            }
        }
        private int Identity
        {
            get
            {
                int iden = Interlocked.Increment(ref _Identity);
                return (iden < 999999999) ? iden : Interlocked.Exchange(ref _Identity, 1);
            }
        }
        public IDbQuery<T> Skip(int index)
        {
            if (string.IsNullOrEmpty(_SortField))
                throw new ApplicationException("need set order filed");
            SqlServerDbMapper oraDbMapper = (SqlServerDbMapper)dbMapper;
            SqlParameter[] parameters = new SqlParameter[dbSQL.Parameters.Count];
            dbSQL.Parameters.CopyTo(parameters, 0);
            SqlServerSQLEntity oraSQL = new SqlServerSQLEntity(
                string.Format("SELECT * FROM (SELECT TABLE_1.*,ROW_NUMBER() OVER(order by {2} {3}) DB_MAPPER_ROWNUM FROM ({0}) TABLE_1) TABLE_2 where TABLE_2.DB_MAPPER_ROWNUM>{1}"
                , dbSQL.SQL,index, _SortField, _SortOrder), SQLType.Select, parameters);
            SqlServerDbQuery<T> queryT = (lstInlineTypes == null)
                ? new SqlServerDbQuery<T>(oraDbMapper, oraSQL)
                : new SqlServerDbQuery<T>(oraDbMapper, oraSQL, lstInlineTypes.ToArray());
            queryT._SortField = _SortField;
            queryT._SortOrder = _SortOrder;
            return queryT;
        }
        public IDbQuery<T> Take(int count)
        {
            if (string.IsNullOrEmpty(_SortField)) 
                throw new ApplicationException("need set order filed");
            SqlServerDbMapper oraDbMapper = (SqlServerDbMapper)dbMapper;
            SqlParameter[] parameters = new SqlParameter[dbSQL.Parameters.Count];
            dbSQL.Parameters.CopyTo(parameters, 0);
            SqlServerSQLEntity oraSQL = new SqlServerSQLEntity(
                string.Format("SELECT TOP {1} * FROM ({0}) TABLE_3"
                ,dbSQL.SQL,count,_SortField,_SortOrder),SQLType.Select,parameters);
            SqlServerDbQuery<T> queryT = (lstInlineTypes == null)
                ? new SqlServerDbQuery<T>(oraDbMapper, oraSQL)
                : new SqlServerDbQuery<T>(oraDbMapper, oraSQL, lstInlineTypes.ToArray());
            queryT._SortField = _SortField;
            queryT._SortOrder = _SortOrder;
            return queryT;

        }
        public IDbQuery<T> OrderBy(string name, bool direct)
        {
            SqlServerDbMapper sqlDbMapper = (SqlServerDbMapper)dbMapper;
            SqlParameter[] parameters = new SqlParameter[dbSQL.Parameters.Count];
            dbSQL.Parameters.CopyTo(parameters, 0);
            _SortField = name;
            _SortOrder=(direct ? "ASC" : "DESC");
            SqlServerSQLEntity sqlSQL = new SqlServerSQLEntity(
                string.Format("SELECT * FROM ({0}) TABLE_4"
                , dbSQL.SQL, name,_SortOrder)
                , SQLType.Select, parameters);
            SqlServerDbQuery<T> queryT=((lstInlineTypes == null)
                ? new SqlServerDbQuery<T>(sqlDbMapper, sqlSQL)
                : new SqlServerDbQuery<T>(sqlDbMapper, sqlSQL, lstInlineTypes.ToArray()));
            queryT._SortField = _SortField;
            queryT._SortOrder = _SortOrder;
            return queryT;
        }
    }
    internal class SqlServerSQLEntity : ISQLEntity
    {
        private string _SQL = null;
        private SQLType _OperType;
        private List<DbParameter> _Parameters = null;
        public string SQL
        {
            get { return _SQL; }
        }
        public List<DbParameter> Parameters
        {
            get { return _Parameters; }
        }
        public SQLType SqlType
        {
            get { return _OperType; }
        }
        public SqlServerSQLEntity(string sql, SQLType type)
        {
            _SQL = sql;
            _OperType = type;
            _Parameters = new List<DbParameter>();
        }
        public SqlServerSQLEntity(string sql, SQLType type, SqlParameter[] parameters)
            : this(sql, type)
        {
            if (parameters.Length > 0)
                _Parameters.AddRange(parameters);
        }
    }
    public class SqlServerDynParameters : Dictionary<string, SqlParameter>, IDbDynParameters
    {
        public T Get<T>(string name)
        {
            if (ContainsKey(name))
            {
                return (T)this[name].Value;
            }
            throw new ApplicationException("name in parameters is not exists");
        }
        public void Add<T>(string name, T value)
        {
            Add<T>(name, value, ParameterDirection.Input);
        }
        public void Add<T>(string name, T value, ParameterDirection direction)
        {
            Add<T>(name, value, direction, -1);
        }
        public void Add<T>(string name, T value, ParameterDirection direction, int size)
        {
            if (ContainsKey(name))
                throw new ApplicationException("name is contain");
            SqlParameter aParameter = new SqlParameter();
            aParameter.Direction = direction;
            aParameter.ParameterName = name;
            if (size > 0) aParameter.Size = size;
            aParameter.Value = value;
            base.Add(name, aParameter);
        }
    }
    #endregion
}