﻿//#define NET4_0

using System;
using System.Collections.Generic;
using System.Text;
using DBMapper;
using System.Data;

namespace DbMapperTest
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                string str_ticks=System.DateTime.Now.Ticks.ToString();
                int jlbh = int.Parse(str_ticks.Substring(str_ticks.Length - 8, 8));
                //---oracle测试
                using (var dbMap = DbMapper.CreateDbMapper(DbMapper.PROVIDER_ORACLE,
                     "Password=test;User ID=test;Data Source=(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL=TCP)"
                     + "(HOST=localhost)(PORT=1521)))(CONNECT_DATA=(SERVER=DEDICATED)(SERVICE_NAME=testdb)));"))
                
                //----sqlserver测试
                //using(var dbMap=DbMapper.CreateDbMapper(DbMapper.PROVIDER_SQLSERVER,"Data Source=DINT-PC;Integrated Security=True; database=TEST"))
                {
                    //插入记录
                    dbMap.Insert<Person>(new Person
                    {
                        JLBH = jlbh,
                        NAME = "索额图",
                        ADDRESS = "皇宫",
                        AGE = 30,
                        PNO = "zzzzzzzzzzzzzzzzz"
                    });
                    
                    dbMap.Insert<Course>(new Course()
                    {
                        JLBH = jlbh,
                        NAME = "计算机科学与技术",
                        TIME = DateTime.Now.ToString()
                    });
                    dbMap.Insert<PersonCourse>(new PersonCourse()
                    {
                        COUR_ID = jlbh,
                        PERSON_ID = jlbh,
                    });
                    dbMap.Execute(false);
                    if(dbMap.Query<Person>("select * from Person where JLBH=:JLBH",new Person{JLBH=jlbh}).Count>0)
                    Console.WriteLine("插入Person记录{0}成功", jlbh);

                    if (dbMap.Query<Course>("select * from Course where JLBH=:JLBH", new Course { JLBH = jlbh }).Count > 0)
                    Console.WriteLine("插入Course记录{0}成功", jlbh);

                    if (dbMap.Query<PersonCourse>("select * from PERSON_COURSE where PERSON_ID=:PERSON_ID", new PersonCourse { PERSON_ID = jlbh }).Count > 0)
                    Console.WriteLine("插入PersonCourse记录{0}成功", jlbh);


                    /*----------------修改--------------------------*/  
                    Console.WriteLine(""); 
                    Console.WriteLine("-----------------修改-----------------");
                    dbMap.Update<Course>(new Course { 
                       JLBH=jlbh,
                       NAME = "计算机科学与技术（修正版）"
                    });
                    int aff=dbMap.Execute();
                    if (aff > 0) {
                        Console.WriteLine("插入Course记录{0}成功", jlbh);
                    }
                    
                    
                    /*----------------单记录查询---------------*/
                    Console.WriteLine(""); 
                    Console.WriteLine("----------------单记录查询---------------");
                    //动态对象传递参数 .NET4.0及以上版本才支持的
                    Person aPerson = dbMap.Query<Person>("select * from Person where JLBH=:JLBH",
                    new {
                        JLBH=jlbh
                    }).First;
                    Console.WriteLine("{0} {1} {2} {3}", aPerson.JLBH, aPerson.NAME, aPerson.PNO, aPerson.ADDRESS);

                    /*------------------------foreach遍历结果集--------------------------------*/
                    Console.WriteLine("");
                    Console.WriteLine("--------foreach遍历结果集-------------");
                    foreach (Person cc in dbMap.Query<Person, Course, PersonCourse>(@"select P.JLBH PERSON#JLBH,P.NAME PERSON#NAME,P.PNO,P.ADDRESS,P.AGE,
                       C.NAME COURSE#NAME,C.TIME from PERSON P,COURSE C,PERSON_COURSE PC
                       where P.JLBH=PC.PERSON_ID and C.JLBH=PC.COUR_ID", null, null, null).OrderBy("PERSON#JLBH", true).Take(5))
                    {
                        Console.WriteLine("{0} {1} {2}", cc.JLBH, cc.NAME, cc.PNO);
                    }

                    /*------------------------------返回结果数量 Count属性-------------------------------------*/
                    Console.WriteLine(""); Console.WriteLine(""); 
                    Console.WriteLine("-------------------返回结果数量 Count属性-----------------");
                    int countPersons = dbMap.Query<Person>("select * from Person", null).Count;
                    Console.WriteLine(countPersons.ToString());

                    /*---------------------------多表查询----------------------------------------*/
                    Console.WriteLine(""); Console.WriteLine("");
                    Console.WriteLine("--------------------多表查询----------------------");
                    
                    DataTable at = dbMap.Query<Object>(@"select P.JLBH PERSON#JLBH,P.NAME PERSON#NAME,P.PNO,P.ADDRESS,P.AGE,
                       C.NAME COURSE#NAME,C.TIME from PERSON P,COURSE C,PERSON_COURSE PC
                       where P.JLBH=PC.PERSON_ID and C.JLBH=PC.COUR_ID", null).OrderBy("PERSON#JLBH", false).Skip(1).Take(5).ToTable();
                    foreach (DataRow arow in at.Rows)
                    {
                        Console.WriteLine("{0} {1} {2} {3} {4}", arow["PERSON#JLBH"], arow["PERSON#NAME"], arow["COURSE#NAME"], arow["TIME"], arow["PNO"]);
                    }


                    /*-------------------------返回结果集-----------------------------*/
                    Console.WriteLine(""); Console.WriteLine(""); 
                    Console.WriteLine("--------------------返回结果集----------------------");
                    List<Person> lstPersons = dbMap.Query<Person>("select * from Person", null).OrderBy("JLBH", true).Skip(0).Take(5).ToList();
                    foreach (Person p in lstPersons)
                    {
                        Console.WriteLine("{0} {1} {2} {3} {4}", p.JLBH, p.NAME, p.PNO, p.AGE, p.ADDRESS);
                    }
                }
            }
            catch(Exception ex){
                Console.WriteLine(ex.Message);
            }
        }
    }

    /*还差一个根据表自动生成对象的工具  这里手动生成*/
    class Person : IDbTableNaming {
        [PrimaryKey]//标示该列为主键  Update 函数需要
        public int JLBH { get; set; }
        public string NAME { get; set; }
        public string PNO { get; set; }
        public string ADDRESS { get; set; }
        public int AGE { get; set; }
        string IDbTableNaming.DBTableName//映射到数据库中的表名  如果没有 则默认为类名
        {
            get { return "PERSON"; }
        }
    }
    class Course : IDbTableNaming {
        [PrimaryKey]
        public int JLBH { get; set; }
        public string NAME { get; set; }
        public string TIME { get; set; }
        string IDbTableNaming.DBTableName
        {
            get { return "COURSE"; }
        }
    }
    class PersonCourse:IDbTableNaming {
        public int PERSON_ID { get; set; }
        public int COUR_ID { get; set; }
        string IDbTableNaming.DBTableName
        {
            get { return "PERSON_COURSE"; }
        }
    }
}
